<?php

namespace Drupal\assembly\Entity;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Entity\RevisionableContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityPublishedTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\user\UserInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Field\FieldDefinitionInterface;

/**
 * Defines the Assembly entity.
 *
 * @ingroup assembly
 *
 * @ContentEntityType(
 *   id = "assembly",
 *   label = @Translation("Assembly"),
 *   bundle_label = @Translation("Assembly type"),
 *   handlers = {
 *     "storage" = "Drupal\assembly\AssemblyStorage",
 *     "view_builder" = "Drupal\assembly\Entity\AssemblyViewBuilder",
 *     "list_builder" = "Drupal\assembly\AssemblyListBuilder",
 *     "compose_widget" = "Drupal\assembly\AssemblyComposeWidget",
 *     "views_data" = "Drupal\assembly\Entity\AssemblyViewsData",
 *     "translation" = "Drupal\assembly\AssemblyTranslationHandler",
 *     "inline_form" = "Drupal\assembly\Form\AssemblyInlineForm",
 *
 *     "form" = {
 *       "default" = "Drupal\assembly\Form\AssemblyForm",
 *       "add" = "Drupal\assembly\Form\AssemblyForm",
 *       "edit" = "Drupal\assembly\Form\AssemblyForm",
 *       "delete" = "Drupal\assembly\Form\AssemblyDeleteForm",
 *     },
 *     "access" = "Drupal\assembly\AssemblyAccessControlHandler",
 *     "route_provider" = {
 *       "html" = "Drupal\assembly\AssemblyHtmlRouteProvider",
 *     },
 *   },
 *   base_table = "assembly",
 *   data_table = "assembly_field_data",
 *   revision_table = "assembly_revision",
 *   revision_data_table = "assembly_field_revision",
 *   show_revision_ui = TRUE,
 *   translatable = TRUE,
 *   admin_permission = "administer assembly entities",
 *   entity_keys = {
 *     "id" = "id",
 *     "revision" = "vid",
 *     "bundle" = "type",
 *     "label" = "name",
 *     "uuid" = "uuid",
 *     "uid" = "user_id",
 *     "langcode" = "langcode",
 *     "status" = "status",
 *   },
 *   revision_metadata_keys = {
 *     "revision_user" = "revision_user",
 *     "revision_created" = "revision_created",
 *     "revision_log_message" = "revision_log",
 *   },
 *   links = {
 *     "canonical" = "/assembly/{assembly}/view",
 *     "add-page" = "/assembly/add",
 *     "add-form" = "/assembly/add/{assembly_type}",
 *     "edit-form" = "/assembly/{assembly}/edit",
 *     "delete-form" = "/assembly/{assembly}/delete",
 *     "version-history" = "/assembly/{assembly}/revisions",
 *     "revision" = "/assembly/{assembly}/revisions/{assembly_revision}/view",
 *     "revision_revert" = "/assembly/{assembly}/revisions/{assembly_revision}/revert",
 *     "translation_revert" = "/assembly/{assembly}/revisions/{assembly_revision}/revert/{langcode}",
 *     "revision_delete" = "/assembly/{assembly}/revisions/{assembly_revision}/delete",
 *     "collection" = "/admin/content/assembly",
 *   },
 *   bundle_entity_type = "assembly_type",
 *   field_ui_base_route = "entity.assembly_type.edit_form"
 * )
 */
class Assembly extends RevisionableContentEntityBase implements AssemblyInterface {

  use EntityChangedTrait;
  use EntityPublishedTrait;

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage_controller, array &$values) {
    parent::preCreate($storage_controller, $values);
    $values += [
      'user_id' => \Drupal::currentUser()->id(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage) {
    parent::preSave($storage);

    foreach (array_keys($this->getTranslationLanguages()) as $langcode) {
      $translation = $this->getTranslation($langcode);

      // If no owner has been set explicitly, make the anonymous user the owner.
      if (!$translation->getOwner()) {
        $translation->setOwnerId(0);
      }
    }

    // If no revision author has been set explicitly, make the assembly owner the
    // revision author.
    if (!$this->getRevisionUser()) {
      $this->setRevisionUserId($this->getOwnerId());
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function urlRouteParameters($rel) {
    $uri_route_parameters = parent::urlRouteParameters($rel);
    if ($rel === 'revision_revert' && $this instanceof RevisionableContentEntityBase) {
      $uri_route_parameters[$this->getEntityTypeId() . '_revision'] = $this->getRevisionId();
    }
    elseif ($rel === 'revision_delete' && $this instanceof RevisionableContentEntityBase) {
      $uri_route_parameters[$this->getEntityTypeId() . '_revision'] = $this->getRevisionId();
    }
    return $uri_route_parameters;
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return $this->get('name')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setName($name) {
    $this->set('name', $name);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    return $this->get('user_id')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->get('user_id')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    $this->set('user_id', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    $this->set('user_id', $account->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function isPublished() {
    return (bool) $this->getEntityKey('status');
  }

  /**
   * {@inheritdoc}
   */
  public function setPublished($published) {
    $this->set('status', $published ? TRUE : FALSE);
    return $this;
  }

  public function getUuid() {
    return $this->uuid->value;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['user_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Authored by'))
      ->setDescription(t('The user ID of author of the Assembly entity.'))
      ->setRevisionable(TRUE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setTranslatable(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Name'))
      ->setDescription(t('Used to identify this assembly in the content management interface. Not visible to site visitors.'))
      ->setRevisionable(TRUE)
      ->setSettings([
        'max_length' => 250,
        'text_processing' => 0,
      ])
      ->setDefaultValueCallback('\Drupal\assembly\Entity\Assembly::getDefaultName')
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => -4,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -4,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['status'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Enabled'))
      ->setDescription(t('A boolean indicating whether the Assembly is published.'))
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'weight' => 99,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setRevisionable(TRUE)
      ->setDefaultValue(TRUE)
      ->setTranslatable(TRUE);

    $fields['visual_styles'] = BaseFieldDefinition::create('list_string')
      ->setLabel(t('Visual styles'))
      ->setRevisionable(TRUE)
      ->setCardinality(FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED)
      ->setSettings([
        'max_length' => 250,
        'text_processing' => 0,
        'allowed_values_function' => 'assembly_allowed_visual_styles',
      ])
      ->setDisplayOptions('form', [
        'type' => 'options_select',
        'multiple' => TRUE,
        'weight' => 1,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDefaultValue('');

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('The time that the entity was created.'));

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the entity was last edited.'));

    $fields['revision_translation_affected'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Revision translation affected'))
      ->setDescription(t('Indicates if the last edit of a translation belongs to current revision.'))
      ->setReadOnly(TRUE)
      ->setRevisionable(TRUE)
      ->setTranslatable(TRUE);

    return $fields;
  }

  static function getDefaultName(FieldableEntityInterface $entity, FieldDefinitionInterface $definition) {
    if ($entity->type->entity) {
      return t('New @label', ['@label' => $entity->type->entity->label()]);
    }
    return 'New assembly';
  }

}
